'''AWS module'''
from .cw import CW
from .ec2 import EC2
from .efs import EFS
from .handler.exception import ExceptionHandler
from .hos import HOS
from .how import HOW
from .iam import IAM
from .instance import INSTANCE_INFO
from .logs import Logs
from .mrktpl import MMetering
from .policy import Policy
from .s3 import S3
from .service import Service
from .sm import SM
from .sqs import SQS
from .sts import STS
from .swf import SWF
